# coding=utf-8
from __future__ import absolute_import, print_function

import logging
import sys
import os
import json
from datetime import date

logger = logging.getLogger(name='naminglog')


def init_logger():
    logger.setLevel(logging.DEBUG)
    # Formatter
    formatter = logging.Formatter(
        '[%(asctime)s:%(module)s:%(funcName)s:'
        '%(lineno)s:%(levelname)s] %(message)s'
    )
    # STDOUT stream
    streamHandler = logging.StreamHandler(sys.stdout)
    streamHandler.setLevel(logging.DEBUG)
    streamHandler.setFormatter(formatter)
    logger.addHandler(streamHandler)


def init_file_logger():
    # Formatter
    formatter = logging.Formatter(
        '[%(asctime)s:%(module)s:%(funcName)s:'
        '%(lineno)s:%(levelname)s] %(message)s'
    )
    # Log file stream
    userPath = os.path.expanduser("~")
    module_dir = os.path.split(__file__)[0]
    config_location = os.path.join(module_dir, "cfg", "config.json")
    config = dict()
    with open(config_location) as fp:
        config = json.load(fp)
    finalDir = os.path.join(userPath, "." + config["logger_dir_name"])

    try:
        if not os.path.exists(finalDir):
            os.mkdir(finalDir)
    except (OSError, IOError) as why:
        raise why

    today = date.today()
    date_string = today.strftime("%d-%m-%Y")
    log_file_path = os.path.join(finalDir, 'naming_{}.log'.format(date_string))
    fileHandler = logging.FileHandler(log_file_path, mode='a')
    fileHandler.setLevel(logging.DEBUG)
    fileHandler.setFormatter(formatter)
    logger.addHandler(fileHandler)



# coding=utf-8
from __future__ import absolute_import, print_function

import copy
import json
import os
from vfxnaming.serialize import Serializable
from vfxnaming.logger import logger

import six

_tokens = dict()


class Token(Serializable):
    def __init__(self, name):
        """Tokens are the meaningful parts of a naming rule. A token can be required,
        meaning fully typed by the user, or can have a set of default options preconfigured.
        If options are present, then one of them is the default one.
        Each option follows a {full_name:abbreviation} schema, so that names can be short
        but meaning can be recovered easily.

        Args:
            name (str): Name that best describes the Token, this will be used as a way
            to invoke the Token object.
        """
        super(Token, self).__init__()
        self._name = name
        self._default = None
        self._options = dict()

    def add_option(self, key, value):
        """Add an option pair to this Token.

        Args:
            key (str): Full length name of the option
            value (str): Abbreviation to be used when building the name.
        """
        self._options[key] = value

    def solve(self, name=None):
        """Solve for abbreviation given a certain name. e.g.: center could return C

        Args:
            name (str, optional): Key to look for in the options for this Token.
                Defaults to None, which will return the default value set in the options
                for this Token.

        Raises:
            Exception: If Token is required and no value is passed.
            Exception: If given name is not found in options list.

        Returns:
            str: If Token is required, the same input value is returned
            str: If Token has options, the abbreviation for given name is returned
            str: If nothing is passed and Token has options, default option is returned.
        """
        if self.required and name:
            return name
        elif self.required and name is None:
            raise Exception("Token {} is required. name parameter must be passed.".format(self.name))
        elif not self.required and name:
            if name not in self._options.keys():
                raise Exception(
                    "name '{}' not found in Token '{}'. Options: {}".format(
                        name, self.name, ', '.join(self._options.keys())
                        )
                    )
            return self._options.get(name)
        elif not self.required and not name:
            return self.default

    def parse(self, value):
        """Get metatada (origin) for given value in name. e.g.: L could return left

        Args:
            value (str): Name part to be parsed to the token origin

        Returns:
            str: Token origin for given value or value itself if no match is found.
        """
        if len(self._options) >= 1:
            for k, v in six.iteritems(self._options):
                if v == value:
                    return k
        return value

    @property
    def required(self):
        return self.default is None

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, n):
        self._name = n

    @property
    def default(self):
        """If Token has options, one of them will be default. Eitther passed by the user,
        or simply the first found item in options.

        Returns:
            str: Default option value
        """
        if self._default is None and len(self._options) >= 1:
            self._default = list(self._options.values())[0]
        return self._default

    @default.setter
    def default(self, d):
        self._default = d

    @property
    def options(self):
        return copy.deepcopy(self._options)


class TokenNumber(Serializable):
    def __init__(self, name):
        """Token for numbers with the ability to handle pure digits and version like strings
        (e.g.: v0025) with padding settings.

        In TokenNumber, options are limited to prefix, suffix and padding.

        Args:
            name (str): Name that best describes the TokenNumber, this will be used as a way
            to invoke the Token object.
        """
        super(TokenNumber, self).__init__()
        self._name = name
        self._default = 1
        self._options = {"prefix": "", "suffix": "", "padding": 3}

    def solve(self, number):
        """Solve for number with prefix, suffix and padding parameter found in the instance
            options. e.g.: 1 with padding 3, will return 001

        Args:
            number (int): Number to be solved for.

        Returns:
            str: The solved string to be used in the name
        """
        numberStr = str(number).zfill(self.padding)
        return '{}{}{}'.format(self.prefix, numberStr, self.suffix)

    def parse(self, value):
        """Get metatada (number) for given value in name. e.g.: v0025 will return 25

        Args:
            value (str): String value taken from a name with digits in it.

        Returns:
            int: Number found in the given string.
        """
        if value.isdigit():
            return int(value)
        else:
            prefix_index = 0
            for each in value[::1]:
                if each.isdigit() and prefix_index == 0:
                    prefix_index = -1
                    break
                elif each.isdigit() and prefix_index > 0:
                    break
                prefix_index += 1

            suffix_index = 0
            for each in value[::-1]:
                if each.isdigit() and suffix_index == 0:
                    suffix_index = -1
                    break
                elif each.isdigit() and suffix_index > 0:
                    break
                suffix_index += 1

            if prefix_index == -1 and suffix_index >= 0:
                return int(value[:-suffix_index])
            elif prefix_index >= 0 and suffix_index == -1:
                return int(value[prefix_index:])
            elif prefix_index >= 0 and suffix_index >= 0:
                return int(value[prefix_index:-suffix_index])

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, n):
        self._name = n

    @property
    def default(self):
        return self._default

    @property
    def required(self):
        return True

    @property
    def padding(self):
        return self._options.get('padding')

    @padding.setter
    def padding(self, p):
        if p <= 0:
            p = 1
        self._options['padding'] = int(p)

    @property
    def prefix(self):
        return self._options.get('prefix')

    @prefix.setter
    def prefix(self, this_prefix):
        if isinstance(this_prefix, str) and not this_prefix.isdigit():
            self._options['prefix'] = this_prefix
        else:
            logger.warning("Given prefix has to be a string: {}".format(this_prefix))

    @property
    def suffix(self):
        return self._options.get('suffix')

    @suffix.setter
    def suffix(self, this_suffix):
        if isinstance(this_suffix, str) and not this_suffix.isdigit():
            self._options['suffix'] = this_suffix
        else:
            logger.warning("Given suffix has to be a string: {}".format(this_suffix))

    @property
    def options(self):
        return copy.deepcopy(self._options)


def add_token(name, **kwargs):
    """Add token to current naming session. If 'default' keyword argument is found,
    set it as default for the token instance.

    Args:
        name (str): Name that best describes the token, this will be used as a way
        to invoke the Token object.

        *kwargs: Each argument following the name is treated as an options for the
        new Token.

    Returns:
        Token: The Token object instance created for given name and fields.
    """
    token = Token(name)
    for k, v in six.iteritems(kwargs):
        if k == "default":
            continue
        token.add_option(k, v)
    if "default" in kwargs.keys():
        token.default = kwargs.get('default')
    _tokens[name] = token
    return token


def add_token_number(name, prefix=str(), suffix=str(), padding=3):
    """Add token number to current naming session.

    Args:
        name (str): Name that best describes the token, this will be used as a way
        to invoke the TokenNumber object.

        prefix (str, optional): Prefix for token number. Useful if you have to use 'v' as prefix
        for versioning for example.

        suffix (str, optional): Suffix for token number.

        padding (int, optional): Padding a numeric value with this leading number of zeroes.
        e.g.: 25 with padding 4 would be 0025

    Returns:
        TokenNumber: The TokenNumber object instance created for given name and fields.
    """
    token = TokenNumber(name)
    token.prefix = prefix
    token.suffix = suffix
    token.padding = padding
    _tokens[name] = token
    return token


def remove_token(name):
    """Remove Token or TokenNumber from current session.

    Args:
        name (str): The name of the token to be removed.

    Returns:
        bool: True if successful, False if a rule name was not found.
    """
    if has_token(name):
        del _tokens[name]
        return True
    return False


def has_token(name):
    """Test if current session has a token with given name.

    Args:
        name (str): The name of the token to be looked for.

    Returns:
        bool: True if rule with given name exists in current session, False otherwise.
    """
    return name in _tokens.keys()


def reset_tokens():
    """Clears all rules created for current session.

    Returns:
        bool: True if clearing was successful.
    """
    _tokens.clear()
    return True


def get_token(name):
    """Gets Token or TokenNumber object with given name.

    Args:
        name (str): The name of the token to query.

    Returns:
        Rule: Token object instance for given name.
    """
    return _tokens.get(name)


def get_tokens():
    """Get all Token and TokenNumber objects for current session.

    Returns:
        dict: {token_name:token_object}
    """
    return _tokens


def save_token(name, filepath):
    """Saves given token serialized to specified location.

    Args:
        name (str): The name of the token to be saved.
        filepath (str): Path location to save the token.

    Returns:
        bool: True if successful, False if rule wasn't found in current session.
    """
    token = get_token(name)
    if not token:
        return False
    with open(filepath, "w") as fp:
        json.dump(token.data(), fp)
    return True


def load_token(filepath):
    """Load token from given location and create Token or TokenNumber object in
    memory to work with it.

    Args:
        filepath (str): Path to existing .token file location

    Returns:
        bool: True if successful, False if .token wasn't found.
    """
    if not os.path.isfile(filepath):
        return False
    try:
        with open(filepath) as fp:
            data = json.load(fp)
    except Exception:
        return False
    class_name = data.get("_Serializable_classname")
    logger.debug("Loading token type: {}".format(class_name))
    token = eval("{}.from_data(data)".format(class_name))
    _tokens[token.name] = token
    return True



# coding=utf-8
from __future__ import absolute_import, print_function

import json
import os
import copy
import platform
from vfxnaming.serialize import Serializable

_separators = dict()


class Separator(Serializable):
    def __init__(self, name):
        """Separators are what splits one token from another. This objects
        allows using multiple system allowed characters as separators. Usually
        separators are '_', '-' or '.', so these are the allowed defaults for
        the object, but you can append to the list. Default symbol is '_'

        Args:
            name (str): Name that best describes the separator.
        """
        super(Separator, self).__init__()
        self._name = name
        self._allowed_symbols = ['_', '-', '.']
        self._symbol = '_'

    def add_allowed_symbol(self, symbol):
        """Add given character symbol to allowed symbols list.

        Args:
            symbol (str): A character separator. e.g.: '_', '-', '.'

        Returns:
            bool: True if symbol was added, False otherwise.
        """
        if symbol not in self._allowed_symbols:
            self._allowed_symbols.append(symbol)
            return True
        return False

    def remove_allowed_symbol(self, symbol):
        """Remove given character symbol from allowed symbols list.

        Args:
            symbol (str): An existing character separator. e.g.: '_', '-', '.'

        Returns:
            bool: True if symbol was removed, False otherwise.
        """
        if symbol in self._allowed_symbols:
            self._allowed_symbols.remove(symbol)
            return True
        return False

    def use_folder_separators(self):
        """"Convenience function to swap file or object naming separators,
        by folder structure separators: '\\' and '/'.

        Symbol will be set to '\\' by default
        """
        self._allowed_symbols = ["\\", "/"]
        this_os = platform.system()
        if this_os == "Windows":
            self._symbol = "\\"
        elif this_os == "Linux":
            self._symbol = "/"
        elif this_os == "Darwin":
            self._symbol = "/"
        else:
            self._symbol = "/"

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, n):
        self._name = n

    @property
    def symbol(self):
        return self._symbol

    @symbol.setter
    def symbol(self, s):
        if s in self._allowed_symbols:
            self._symbol = s

    @property
    def allowed_symbols(self):
        return copy.deepcopy(self._allowed_symbols)


def add_separator(name, symbol='_'):
    """Add separator to current naming session.

    Args:
        name (str): Name that best describes the separator
        symbol (str, optional): Symbol to be used as separator.
        Tip: Use a single character. Defaults to '_'.

    Returns:
        Separator: The Separator object instance created.
    """
    separator = Separator(name)
    separator.symbol = symbol
    _separators[name] = separator
    return separator


def remove_separator(name):
    """Remove Separator from current session.

    Args:
        name (str): The name of the separator to be removed.

    Returns:
        bool: True if successful, False if a separator name was not found.
    """
    if has_separator(name):
        del _separators[name]
        return True
    return False


def has_separator(name):
    """Test if current session has a separator with given name.

    Args:
        name (str): The name of the separator to be looked for.

    Returns:
        bool: True if separator with given name exists in current session, False otherwise.
    """
    return name in _separators.keys()


def reset_separators():
    """Clears all separators created for current session.

    Returns:
        bool: True if clearing was successful.
    """
    _separators.clear()
    return True


def get_separator(name):
    """Gets Separator object with given name.

    Args:
        name (str): The name of the separator to query.

    Returns:
        Separator: Separator object instance for given name.
    """
    return _separators.get(name)


def get_separators():
    """Get all Separator objects for current session.

    Returns:
        dict: {separator_name:Separator}
    """
    return _separators


def save_separator(name, filepath):
    """Saves given separator serialized to specified location.

    Args:
        name (str): The name of the separator to be saved.
        filepath (str): Path location to save the separator.

    Returns:
        bool: True if successful, False if separator wasn't found in current session.
    """
    token = get_separator(name)
    if not token:
        return False
    with open(filepath, "w") as fp:
        json.dump(token.data(), fp)
    return True


def load_separator(filepath):
    """Load separator from given location and create Separator object in memory to
    work with it.

    Args:
        filepath (str): Path to existing .separator file location

    Returns:
        bool: True if successful, False if .separator wasn't found.
    """
    if not os.path.isfile(filepath):
        return False
    try:
        with open(filepath) as fp:
            data = json.load(fp)
    except Exception:
        return False
    separator = Separator.from_data(data)
    _separators[separator.name] = separator
    return True



# coding=utf-8
from __future__ import absolute_import, print_function

import re
import json
import os
from vfxnaming.serialize import Serializable
from vfxnaming.separators import get_separators
from vfxnaming.tokens import get_token
from vfxnaming.logger import logger

import six

_rules = {'_active': None}


class Rule(Serializable):
    def __init__(self, name):
        """Each rule is managed by an instance of this class. Fields exist for each
        Token and Separator used in the rule definition.

        Args:
            name (str): Name that best describes the rule, this will be used as a way
            to invoke the Rule object.
        """
        super(Rule, self).__init__()
        self.name = name
        self._fields = list()

    def add_fields(self, token_names):
        """Add fields to rule, by appending them to already existing ones.

        Args:
            token_names (list): List of new field names.

        Returns:
            bool: True if operation was successful
        """
        self._fields.extend(token_names)
        return True

    def solve(self, **values):
        """Given arguments are used to build a name.

        Returns:
            str: A string with the resulting name.
        """
        result = None
        try:
            # Try to solve with given values
            result = self.pattern.format(**values)
        except KeyError:
            # If KeyError, then separators are part of the rule and need to be added
            symbols_dict = dict()
            for name, separator in six.iteritems(get_separators()):
                symbols_dict[name] = separator.symbol
            values.update(symbols_dict)
            result = self.pattern.format(**values)
        return result

    def parse(self, name):
        """Build and return dictionary with keys as tokens and values as given names.

        Args:
            name (str): Name string e.g.: C_helmet_001_MSH

        Returns:
            dict: A dictionary with keys as tokens and values as given name parts.
            e.g.: {'side':'C', 'part':'helmet', 'number': 1, 'type':'MSH'}
        """
        delimiters = [value.symbol for key, value in six.iteritems(get_separators())]
        if len(delimiters) >= 1:
            logger.debug("Parsing with these separators: {}".format(', '.join(delimiters)))
            regex_pattern = '(' + '|'.join(map(re.escape, delimiters)) + ')'
            name_parts = re.split(regex_pattern, name)
            retval = dict()
            for i, f in enumerate(self.fields):
                name_part = name_parts[i]
                token = get_token(f)
                if not token:
                    continue
                retval[f] = token.parse(name_part)
            return retval
        logger.warning(
            "No separators used for rule {}, parsing is not possible.".format(
                self.name)
            )
        return None

    @property
    def pattern(self):
        return '{' + '}{'.join(self.fields) + '}'

    @property
    def fields(self):
        return tuple(self._fields)

    @fields.setter
    def fields(self, f):
        self._fields = f

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, n):
        self._name = n


class TemplateRule(Rule):
    """Useful for folder naming.
    Using \\ as separator (and removing defaults),
    this rule will assume not passed field values as values themselves. So
    'harcoded' values, so common in folder structure naming, can also be solved.
    e.g.: {project_name}\\{CFG} with passed values project_name=my_project, will
    return my_project\\CFG
    """
    def __init__(self, name):
        super(TemplateRule, self).__init__(name)

    def solve(self, **values):
        """Given arguments are used to build a name. If no value is specified,
        the field name itself is used as value.

        Returns:
            str: A string with the resulting name.
        """
        result = None
        symbols_dict = dict()
        for name, separator in six.iteritems(get_separators()):
            symbols_dict[name] = separator.symbol
        values.update(symbols_dict)

        for f in self.fields:
            if f not in values.keys():
                values[f] = f

        result = self.pattern.format(**values)
        return result


def add_template_rule(name, *fields):
    """Add template rule to current naming session. If no active rule is found, it adds
    the created one as active by default.

    Args:
        name (str): Name that best describes the template rule, this will be used as a way
        to invoke the TemplateRule object.

        fields: Each argument following the name is treated as a field for the
        new TemplateRule

    Returns:
        Rule: The TemplateRule object instance created for given name and fields.
    """
    rule = TemplateRule(name)
    rule.fields = fields
    _rules[name] = rule
    if get_active_rule() is None:
        set_active_rule(name)
        logger.debug("No active rule found, setting this one as active: {}".format(name))
    return rule


def add_rule(name, *fields):
    """Add rule to current naming session. If no active rule is found, it adds
    the created one as active by default.

    Args:
        name (str): Name that best describes the rule, this will be used as a way
        to invoke the Rule object.

        fields: Each argument following the name is treated as a field for the
        new Rule

    Returns:
        Rule: The Rule object instance created for given name and fields.
    """
    rule = Rule(name)
    rule.fields = fields
    _rules[name] = rule
    if get_active_rule() is None:
        set_active_rule(name)
        logger.debug("No active rule found, setting this one as active: {}".format(name))
    return rule


def remove_rule(name):
    """Remove Rule from current session.

    Args:
        name (str): The name of the rule to be removed.

    Returns:
        bool: True if successful, False if a rule name was not found.
    """
    if has_rule(name):
        del _rules[name]
        return True
    return False


def has_rule(name):
    """Test if current session has a rule with given name.

    Args:
        name (str): The name of the rule to be looked for.

    Returns:
        bool: True if rule with given name exists in current session, False otherwise.
    """
    return name in _rules.keys()


def reset_rules():
    """Clears all rules created for current session.

    Returns:
        bool: True if clearing was successful.
    """
    _rules.clear()
    _rules['_active'] = None
    return True


def get_active_rule():
    """Get currently active rule for the session. This is the rule
    that will be used to parse and solve from.

    Returns:
        Rule: Rule object instance for currently active Rule.
    """
    name = _rules.get('_active')
    return _rules.get(name)


def set_active_rule(name):
    """Sets given rule as active for the session. This it the rule that's
    being used to parse and solve from.

    Args:
        name (str): The name of the rule to be activated.

    Returns:
        bool: True if successful, False otherwise.
    """
    if has_rule(name):
        _rules['_active'] = name
        return True
    return False


def get_rule(name):
    """Gets Rule object with given name.

    Args:
        name (str): The name of the rule to query.

    Returns:
        Rule: Rule object instance for given name.
    """
    return _rules.get(name)


def get_rules():
    """Get all Rule objects for current session.

    Returns:
        dict: {rule_name:Rule}
    """
    return _rules


def save_rule(name, filepath):
    """Saves given rule serialized to specified location.

    Args:
        name (str): The name of the rule to be saved.
        filepath (str): Path location to save the rule.

    Returns:
        bool: True if successful, False if rule wasn't found in current session.
    """
    rule = get_rule(name)
    if not rule:
        return False
    with open(filepath, "w") as fp:
        json.dump(rule.data(), fp)
    return True


def load_rule(filepath):
    """Load rule from given location and create Rule object in memory to work with it.

    Args:
        filepath (str): Path to existing .rule file location

    Returns:
        bool: True if successful, False if .rule wasn't found.
    """
    if not os.path.isfile(filepath):
        return False
    try:
        with open(filepath) as fp:
            data = json.load(fp)
    except Exception:
        return False
    rule = Rule.from_data(data)
    _rules[rule.name] = rule
    return True



# coding=utf-8
# MIT License
# Copyright (c) 2017 Cesar Saez and modified by Chris Granados- Xian
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from __future__ import absolute_import, print_function

import os
import json
import vfxnaming.rules as rules
import vfxnaming.tokens as tokens
import vfxnaming.separators as separators
from vfxnaming.logger import logger

import six

NAMING_REPO_ENV = "NAMING_REPO"


def parse(name):
    """Get metadata from a name string recognized by the currently active rule.

    Args:
        name (str): Name string e.g.: C_helmet_001_MSH

    Returns:
        dict: A dictionary with keys as tokens and values as given name parts.
        e.g.: {'side':'C', 'part':'helmet', 'number': 1, 'type':'MSH'}
    """
    rule = rules.get_active_rule()
    return rule.parse(name)


def solve(*args, **kwargs):
    """Given arguments are used to build a name following currently active rule.

    Raises:
        Exception: A required token was passed as None to keyword arguments.
        IndexError: Missing argument for one field in currently active rule.

    Returns:
        str: A string with the resulting name.
    """
    values = dict()
    rule = rules.get_active_rule()
    i = 0
    for f in rule.fields:
        separator = separators.get_separator(f)
        if separator:
            continue
        token = tokens.get_token(f)
        if token:
            # Explicitly passed as keyword argument
            if kwargs.get(f) is not None:
                values[f] = token.solve(kwargs.get(f))
                continue
            elif token.required and kwargs.get(f) is None and len(args) == 0:
                raise Exception("Token {} is required.")
            elif not token.required and kwargs.get(f) is None:
                values[f] = token.solve()
                continue
            # Implicitly passed as positional argument
            try:
                values[f] = token.solve(args[i])
                i += 1
                continue
            except IndexError as why:
                raise IndexError("Missing argument for field '{}'\n{}".format(f, why))
    logger.debug("Solving rule {} with values {}".format(rule.name, values))
    return rule.solve(**values)


def get_repo():
    """Get repository location from either global environment variable or local user,
    giving priority to environment variable.

    Environment varialble name: NAMING_REPO

    Returns:
        str: Naming repository location
    """
    env_repo = os.environ.get(NAMING_REPO_ENV)
    userPath = os.path.expanduser("~")
    module_dir = os.path.split(__file__)[0]
    config_location = os.path.join(module_dir, "cfg", "config.json")
    config = dict()
    with open(config_location) as fp:
        config = json.load(fp)
    local_repo = os.path.join(userPath, "." + config["local_repo_name"], "naming_repo")
    result = env_repo or local_repo
    logger.debug("Repo found: {}".format(result))
    return result


def save_session(repo=None):
    """Save rules, tokens, separators and config files to the repository.

    Raises:
        IOError, OSError: Repository directory could not be created.

    Args:
        repo (str, optional): Absolue path to a repository. Defaults to None.

    Returns:
        bool: True if saving session operation was successful.
    """
    repo = repo or get_repo()
    if not os.path.exists(repo):
        try:
            os.mkdir(repo)
        except (IOError, OSError) as why:
            raise why
    # save tokens
    for name, token in six.iteritems(tokens.get_tokens()):
        filepath = os.path.join(repo, name + ".token")
        logger.debug("Saving token: {} in {}".format(name, filepath))
        tokens.save_token(name, filepath)
    # save rules
    for name, rule in six.iteritems(rules.get_rules()):
        if not isinstance(rule, rules.Rule):
            continue
        filepath = os.path.join(repo, name + ".rule")
        logger.debug("Saving rule: {} in {}".format(name, filepath))
        rules.save_rule(name, filepath)
    # save separators
    for name, separator in six.iteritems(separators.get_separators()):
        filepath = os.path.join(repo, name + ".separator")
        logger.debug("Saving separator: {} in {}".format(name, filepath))
        separators.save_separator(name, filepath)
    # extra configuration
    active = rules.get_active_rule()
    config = {"set_active_rule": active.name if active else None}
    filepath = os.path.join(repo, "naming.conf")
    logger.debug("Saving active rule: {} in {}".format(active.name, filepath))
    with open(filepath, "w") as fp:
        json.dump(config, fp, indent=4)
    return True


def load_session(repo=None):
    """Load rules, tokens, separators and config from a repository, and create
    Python objects in memory to work with them.

    Args:
        repo (str, optional): Absolute path to a repository. Defaults to None.

    Returns:
        bool: True if loading session operation was successful.
    """
    repo = repo or get_repo()
    if not os.path.exists(repo):
        logger.warning("Given repo directory does not exist: {}".format(repo))
        return False
    # tokens, rules and separators
    for dirpath, dirnames, filenames in os.walk(repo):
        for filename in filenames:
            filepath = os.path.join(dirpath, filename)
            if filename.endswith(".token"):
                logger.debug("Loading token: {}".format(filepath))
                tokens.load_token(filepath)
            elif filename.endswith(".rule"):
                logger.debug("Loading rule: {}".format(filepath))
                rules.load_rule(filepath)
            elif filename.endswith(".separator"):
                logger.debug("Loading separator: {}".format(filepath))
                separators.load_separator(filepath)
    # extra configuration
    filepath = os.path.join(repo, "naming.conf")
    if os.path.exists(filepath):
        logger.debug("Loading active rule: {}".format(filepath))
        with open(filepath) as fp:
            config = json.load(fp)
        rules.set_active_rule(config.get('set_active_rule'))
    return True



# coding=utf-8
from __future__ import absolute_import, print_function

from vfxnaming.naming import parse, solve, get_repo, save_session, load_session
from vfxnaming.rules import add_rule, add_template_rule, remove_rule, has_rule, reset_rules, get_active_rule, set_active_rule, get_rule, get_rules, save_rule, load_rule
from vfxnaming.tokens import add_token, add_token_number, remove_token, has_token, reset_tokens, get_token, get_tokens, save_token, load_token
from vfxnaming.separators import add_separator, remove_separator, has_separator, reset_separators, get_separator, get_separators, save_separator, load_separator



from setuptools import setup, find_packages
from os import path
from io import open

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='vfxnaming',
    version='1.1.4-beta',
    description='Naming conventions library',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/xiancg/vfxnaming',
    download_url='https://github.com/xiancg/vfxnaming/archive/v1.1.4-beta.tar.gz',
    author='Cesar Saez, Chris Granados- Xian',
    author_email='info@chrisgranados.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.7'
    ],
    package_dir={'': 'src'},
    packages=find_packages(where='src'),
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, <4',
    install_requires=['six'],
    extras_require={
        'dev': ['pytest', 'pytest-cov', 'pytest-datafiles', 'python-coveralls', 'flake8'],
        'docs': ['sphinx', 'sphinx-rtd-theme']
    },
    package_data={'': ['cfg/config.json']}
)
